@@ -1,3 +1,9 @@
+# -*- coding: utf-8 -*-
+# @Author: gviejo
+# @Date:   2022-01-27 18:33:31
+# @Last Modified by:   gviejo
+# @Last Modified time: 2022-04-07 11:24:31
+
 import pandas as pd
 import numpy as np
 import warnings
@@ -5,7 +11,6 @@ from .time_units import TimeUnits
 from pandas.core.internals import SingleBlockManager, BlockManager
 from .interval_set import IntervalSet
 
-
 def _get_restrict_method(align):
     """
     Get method for alignment
@@ -53,9 +58,8 @@ def support_func(data, min_gap, method='absolute'):
     support_here = span.set_diff(here_gaps)
     return support_here
 
-
 class Tsd(pd.Series):
-# class Tsd():
+    # class Tsd():
     """
     A subclass of pandas.Series specialized for neurophysiology time series.
     
@@ -69,7 +73,7 @@ class Tsd(pd.Series):
         The time support of the time series
     """
     
-    def __init__(self, t, d=None, time_units=None, time_support=None, **kwargs):
+    def __init__(self, t, d=None, time_units='s', time_support=None, **kwargs):
         """
         Tsd Initializer.
                 
@@ -80,7 +84,7 @@ class Tsd(pd.Series):
         d : numpy.ndarray, optional
             The data of the time series
         time_units : str, optional
-            The time units in which times are specified (in s, ms or us)
+            The time units in which times are specified ('us', 'ms', 's' [default])
         time_support : IntervalSet, optional
             The time support of the tsd object
         **kwargs
@@ -96,40 +100,69 @@ class Tsd(pd.Series):
 
         t = TimeUnits.format_timestamps(t, time_units)
 
-        if time_support is not None:
-            bins = time_support.values.ravel()
-            ix = np.array(pd.cut(t, bins, labels=np.arange(len(bins) - 1, dtype=np.float64)))
-            ix[np.floor(ix / 2) * 2 != ix] = np.NaN
-            ix = np.floor(ix/2)
-            ix = ~np.isnan(ix)
-            if d is not None:
-                super().__init__(index=t[ix],data=d[ix])
+        if len(t):
+            if time_support is not None:
+                bins = time_support.values.ravel()
+                # Because yes there is no funtion with both bounds closed as an option
+                ix = np.vstack((
+                    np.array(pd.cut(t, bins, labels=np.arange(len(bins) - 1, dtype=np.float64))),
+                    np.array(pd.cut(t, bins, labels=np.arange(len(bins) - 1, dtype=np.float64), right=False))
+                    )).T
+                ix[np.floor(ix / 2) * 2 != ix] = np.NaN
+                ix = np.floor(ix/2)
+                ix[np.isnan(ix[:,0]),0] = ix[np.isnan(ix[:,0]),1]
+                ix = ~np.isnan(ix[:,0])
+                if d is not None:
+                    super().__init__(index=t[ix], data=d[ix])
+                else:
+                    super().__init__(index=t[ix], data=None, dtype=np.float64)
             else:
-                super().__init__(index=t[ix],data=None)
+                time_support = IntervalSet(start = t[0], end = t[-1], time_units = 's')
+                if d is not None:
+                    super().__init__(index=t, data=d)
+                else:
+                    super().__init__(index=t, data=d, dtype=np.float64)
         else:
-            time_support = IntervalSet(start = t[0], end = t[-1])
-            super().__init__(index=t, data=d)
+            time_support = IntervalSet(pd.DataFrame(columns=['start','end']))
+            super().__init__(index=t, data=d, dtype=np.float64)
 
         self.time_support = time_support
         self.rate = len(t)/self.time_support.tot_length('s')
-        self.index.name = "Time (us)"
-        self._metadata.append("nts_class")
-        self.nts_class = self.__class__.__name__
+        self.index.name = "Time (s)"
+        self._metadata.append("nap_class")
+        self.nap_class = self.__class__.__name__
 
+    def __lt__(self, value):
+        return self.as_series().__lt__(value)
+
+    def __gt__(self, value):
+        return self.as_series().__gt__(value)
+
+    def __le__(self, value):
+        return self.as_series().__le__(value)
+
+    def __ge__(self, value):
+        return self.as_series().__ge__(value)
+
+    def __ne__(self, value):
+        return self.as_series().__ne__(value)
+
+    def __eq__(self, value):        
+        return self.as_series().__eq__(value)
 
     def __repr__(self):
-        return self.as_units('s').__repr__()
+        return self.as_series().__repr__()
 
     def __str__(self): return self.__repr__()
 
-    def times(self, units=None):
+    def times(self, units='s'):
         """
         The time index of the Tsd, returned as np.double in the desired time units.
         
         Parameters
         ----------
         units : str, optional
-            us, ms, or s (default is us)
+            ('us', 'ms', 's' [default])
 
         Returns
         -------
@@ -150,14 +183,14 @@ class Tsd(pd.Series):
         """
         return pd.Series(self, copy=True)
 
-    def as_units(self, units=None):
+    def as_units(self, units='s'):
         """
         Returns a Series with time expressed in the desired unit.
         
         Parameters
         ----------
         units : str, optional
-            us, ms, or s (default is us)
+            ('us', 'ms', 's' [default])
         
         Returns
         -------
@@ -167,10 +200,12 @@ class Tsd(pd.Series):
         ss = self.as_series()
         t = self.index.values
         t = TimeUnits.return_timestamps(t, units)
+        if units == 'us':
+            t = t.astype(np.int64)
         ss.index = t
         units_str = units
         if not units_str:
-            units_str = 'us'
+            units_str = 's'
         ss.index.name = "Time (" + units_str + ")"
         return ss
 
@@ -185,7 +220,7 @@ class Tsd(pd.Series):
         """
         return self.values
 
-    def value_from(self, tsd, ep, align='closest'):
+    def value_from(self, tsd, ep=None, align='closest'):
         """
         Replace the value with the closest value from tsd argument
         
@@ -193,20 +228,21 @@ class Tsd(pd.Series):
         ----------
         tsd : Tsd
             The Tsd object holding the values to replace
-        ep : IntervalSet
-            The IntervalSet object to restrict the operation
+        ep : IntervalSet (optional)
+            The IntervalSet object to restrict the operation. 
+            If None, the time support of the tsd input object is used.
         align : str, optional
             The method to align (closest/prev/next)
         
         Returns
         -------
-        out: Tsd
+        out : Tsd
             Tsd object with the new values
         
         Example
         -------
         In this example, the ts object will receive the closest values in time from tsd.
-
+        
         >>> import pynapple as nap
         >>> import numpy as np
         >>> t = np.unique(np.sort(np.random.randint(0, 1000, 100))) # random times
@@ -224,6 +260,8 @@ class Tsd(pd.Series):
         >>> print(len(ts.restrict(ep)), len(newts))
             52 52
         """
+        if ep is None:
+            ep = tsd.time_support
         method = _get_restrict_method(align)
         ix = TimeUnits.format_timestamps(self.restrict(ep).index.values)
         tsd = tsd.restrict(ep)
@@ -231,7 +269,7 @@ class Tsd(pd.Series):
         new_tsd = tsd.reindex(ix, method=method)
         return Tsd(new_tsd, time_support = ep)
 
-    def restrict(self, ep, keep_labels=False):
+    def restrict(self, ep):
         """
         Restricts a Tsd object to a set of time intervals delimited by an IntervalSet object
         
@@ -239,8 +277,6 @@ class Tsd(pd.Series):
         ----------
         ep : IntervalSet
             the IntervalSet object 
-        keep_labels : bool, optional
-            Wheter or not to drop the label of a column
         
         Returns
         -------
@@ -288,7 +324,7 @@ class Tsd(pd.Series):
             IntervalSet to restrict the operation
         
         time_units : str, optional
-            Time units of bin size (us, ms, s)
+            Time units of bin size ('us', 'ms', 's' [default])
         
         Returns
         -------
@@ -319,13 +355,13 @@ class Tsd(pd.Series):
         if not isinstance(ep, IntervalSet):
             ep = self.time_support
             
-        bin_size_us = TimeUnits.format_timestamps(np.array([bin_size]), time_units)[0]
+        bin_size = TimeUnits.format_timestamps(np.array([bin_size]), time_units)[0]
 
         # bin for each epochs
         time_index = []
         count = []
         for i in ep.index:
-            bins = np.arange(ep.start[i], ep.end[i] + bin_size_us, bin_size_us)
+            bins = np.arange(ep.start[i], ep.end[i] + bin_size, bin_size)
             count.append(np.histogram(self.index.values, bins)[0])
             time_index.append(bins[0:-1] + np.diff(bins)/2)
         time_index = np.hstack(time_index)
@@ -403,21 +439,20 @@ class Tsd(pd.Series):
             tsd = self.restrict(time_support)
             return tsd
 
-
     def gaps(self, min_gap, method='absolute'):
         return gaps_func(self, min_gap, method)
 
     def support(self, min_gap, method='absolute'):
         return support_func(self, min_gap, method)
 
-    def start_time(self, units='us'):
+    def start_time(self, units='s'):
         """
         The first time index in the Ts/Tsd object
         
         Parameters
         ----------
         units : str, optional
-            us, ms or s (default is us)
+            ('us', 'ms', 's' [default])
         
         Returns
         -------
@@ -426,14 +461,14 @@ class Tsd(pd.Series):
         """        
         return self.times(units=units)[0]
 
-    def end_time(self, units='us'):
+    def end_time(self, units='s'):
         """
         The last time index in the Ts/Tsd object
         
         Parameters
         ----------
         units : str, optional
-            us, ms or s (default is us)
+            ('us', 'ms', 's' [default])
         
         Returns
         -------
@@ -449,7 +484,7 @@ class Tsd(pd.Series):
 
 # noinspection PyAbstractClass
 class TsdFrame(pd.DataFrame):
-# class TsdFrame():
+    # class TsdFrame():
     """
     A subclass of pandas.DataFrame specialized for neurophysiological time series.
     
@@ -463,7 +498,7 @@ class TsdFrame(pd.DataFrame):
         The time support of the time series
     """
     
-    def __init__(self, t, d=None, time_units=None, time_support=None, **kwargs):
+    def __init__(self, t, d=None, time_units='s', time_support=None, **kwargs):
         """
         TsdFrame initializer
         
@@ -474,7 +509,7 @@ class TsdFrame(pd.DataFrame):
         d : numpy.ndarray
             The data
         time_units : str, optional
-            The time units in which times are specified
+            The time units in which times are specified ('us', 'ms', 's' [default]).
         time_support : IntervalSet, optional
             The time support of the TsdFrame object
         **kwargs
@@ -494,10 +529,15 @@ class TsdFrame(pd.DataFrame):
 
         if time_support is not None:
             bins = time_support.values.ravel()
-            ix = np.array(pd.cut(t, bins, labels=np.arange(len(bins) - 1, dtype=np.float64)))
+            # Because yes there is no funtion with both bounds closed as an option
+            ix = np.vstack((
+                np.array(pd.cut(t, bins, labels=np.arange(len(bins) - 1, dtype=np.float64))),
+                np.array(pd.cut(t, bins, labels=np.arange(len(bins) - 1, dtype=np.float64), right=False))
+                )).T
             ix[np.floor(ix / 2) * 2 != ix] = np.NaN
             ix = np.floor(ix/2)
-            ix = ~np.isnan(ix)
+            ix[np.isnan(ix[:,0]),0] = ix[np.isnan(ix[:,0]),1]
+            ix = ~np.isnan(ix[:,0])
             super().__init__(index=t[ix],data=d[ix], columns = c)
         else:
             time_support = IntervalSet(start = t[0], end = t[-1])
@@ -508,9 +548,9 @@ class TsdFrame(pd.DataFrame):
             self.time_support = time_support
 
         self.rate = len(t)/self.time_support.tot_length('s')
-        self.index.name = "Time (us)"
-        self._metadata.append("nts_class")
-        self.nts_class = self.__class__.__name__
+        self.index.name = "Time (s)"
+        self._metadata.append("nap_class")
+        self.nap_class = self.__class__.__name__
 
     def __repr__(self):
         return self.as_units('s').__repr__()
@@ -525,14 +565,14 @@ class TsdFrame(pd.DataFrame):
         elif isinstance(result, pd.DataFrame):
             return TsdFrame(result, time_support=time_support)
 
-    def times(self, units=None):
+    def times(self, units='s'):
         """
         The time index of the TsdFrame, returned as np.double in the desired time units.
         
         Parameters
         ----------
         units : str, optional
-            us, ms, or s (default is us)
+            ('us', 'ms', 's' [default])
         
         Returns
         -------
@@ -552,14 +592,14 @@ class TsdFrame(pd.DataFrame):
         """
         return pd.DataFrame(self, copy=copy)
 
-    def as_units(self, units=None):
+    def as_units(self, units='s'):
         """
         Returns a DataFrame with time expressed in the desired unit.
         
         Parameters
         ----------
         units : str, optional
-            us, ms, or s (default is us)
+            ('us', 'ms', 's' [default])
         
         Returns
         -------
@@ -568,10 +608,13 @@ class TsdFrame(pd.DataFrame):
         """
         t = self.index.values.copy()
         t = TimeUnits.return_timestamps(t, units)
+        if units == 'us':
+            t = t.astype(np.int64)
+
         df = pd.DataFrame(index=t, data=self.values)
         units_str = units
         if not units_str:
-            units_str = 'us'
+            units_str = 's'
         df.index.name = "Time (" + units_str + ")"
         df.columns = self.columns.copy()
         return df
@@ -596,6 +639,55 @@ class TsdFrame(pd.DataFrame):
         rest_t = self.reindex(ix, method=method, columns=self.columns.values)
         return rest_t
 
+    def value_from(self, tsd, ep=None, align='closest'):
+        """
+        Replace the value with the closest value from tsd argument
+        
+        Parameters
+        ----------
+        tsd : Tsd
+            The Tsd object holding the values to replace
+        ep : IntervalSet, optional
+            The IntervalSet object to restrict the operation.
+            If None, ep is taken from the tsd of the time support
+        align : str, optional
+            The method to align (closest/prev/next)
+        
+        Returns
+        -------
+        out: Tsd
+            Tsd object with the new values
+        
+        Example
+        -------
+        In this example, the ts object will receive the closest values in time from tsd.
+
+        >>> import pynapple as nap
+        >>> import numpy as np
+        >>> t = np.unique(np.sort(np.random.randint(0, 1000, 100))) # random times
+        >>> ts = nap.Ts(t=t, time_units='s')
+        >>> tsd = nap.TsdFrame(t=np.arange(0,1000), d=np.random.rand(1000), time_units='s')
+        >>> ep = nap.IntervalSet(start = 0, end = 500, time_units = 's')
+        
+        The variable ts is a time series object containing only nan.
+        The tsd object containing the values, for example the tracking data, and the epoch to restrict the operation.
+        
+        >>> newts = ts.value_from(tsd, ep)
+        
+        newts is the same size as ts restrict to ep.
+        
+        >>> print(len(ts.restrict(ep)), len(newts))
+            52 52
+        """
+        if ep is None:
+            ep = tsd.time_support
+        method = _get_restrict_method(align)
+        ix = TimeUnits.format_timestamps(self.restrict(ep).index.values)
+        tsd = tsd.restrict(ep)
+        tsd = tsd.as_series()
+        new_tsd = tsd.reindex(ix, method=method)
+        return Tsd(new_tsd, time_support = ep)
+
     def restrict(self, iset, keep_labels=False):
         """
         Restricts a TsdFrame object to a set of time intervals delimited by an IntervalSet object`
@@ -628,10 +720,10 @@ class TsdFrame(pd.DataFrame):
     def support(self, min_gap, method='absolute'):
         return support_func(self, min_gap, method)
 
-    def start_time(self, units='us'):
+    def start_time(self, units='s'):
         return self.times(units=units)[0]
 
-    def end_time(self, units='us'):
+    def end_time(self, units='s'):
         return self.times(units=units)[-1]
 
 
@@ -650,7 +742,7 @@ class Ts(Tsd):
         The time support of the time series
     """
 
-    def __init__(self, t, time_units=None, time_support=None,**kwargs):
+    def __init__(self, t, time_units='s', time_support=None,**kwargs):
         """
         Ts Initializer
         
@@ -659,12 +751,15 @@ class Ts(Tsd):
         t : numpy.ndarray or pandas.Series
             An object transformable in a time series, or a pandas.Series equivalent (if d is None)
         time_units : str, optional
-            The time units in which times are specified (in s, ms or us)
+            The time units in which times are specified ('us', 'ms', 's' [default])
         time_support : IntervalSet, optional
             The time support of the Ts object
         **kwargs
             Arguments that will be passed to the pandas.Series initializer.
         """
-        super().__init__(t, None, time_units=time_units, time_support=time_support, **kwargs)
+        super().__init__(t, None, 
+            time_units=time_units, 
+            time_support=time_support, 
+            dtype=np.float64,**kwargs)
         self.nts_class = self.__class__.__name__
 
